package org.daligo.app;

import java.io.File;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.daligo.domain.DiaryDBAdapter;
import org.daligo.domain.GeopointDBAdapter;
import org.daligo.domain.PathOverlay;
import org.daligo.domain.ProfileDBAdapter;
import org.daligo.type.DaligoPath;
import org.daligo.type.RunningData;
import org.daligo.type.Status;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.database.Cursor;
import android.graphics.Color;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.DeadObjectException;
import android.os.Handler;
import android.os.IBinder;
import android.os.RemoteException;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.util.Log;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;

public class GoBasicActivity extends MapActivity implements OnInitListener {

	private ProfileDBAdapter profileDBAdapter;
	private Cursor profileCursor;

	// related to GPS signal
	private LocationManager lm;
	private LocationListener locationListener;

	// related to MapView
	private MapView googleMap;
	private MapController mc;
	private DaligoPath myPath;
	protected Status status;

	protected DecimalFormat f = new DecimalFormat("0.00");
	protected TextView distanceText;
	protected TextView elapsedText;
	protected TextView speedText;
	protected TextView caloriesText;

	// context menu
	private static final int PAUSE_RESUME = 0;
	private static final int PAUSE_SAVE = 1;
	private static final int PAUSE_EXIT = 2;

	// related to Interface
	private Handler outerHandler = new Handler();
	private Handler innerHandler = new Handler();

	// 시계를 위한 Runnable
	private Runnable runnable = new Runnable() {
		@Override
		public void run() {
			status.tick();
			status.update();
			updateText();
			googleMap.postInvalidate();
			innerHandler.postDelayed(this, 1000);
		}
	};

	public static final String MEDIA_PATH = new String("/sdcard/");
	private List<String> songs = new ArrayList<String>();
	private MusicPlayerInterface mpInterface;

	public TextToSpeech mTts;
	public String Information;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		Toast.makeText(GoBasicActivity.this, "onCreate", Toast.LENGTH_SHORT)
				.show();
		super.onCreate(savedInstanceState);
		setContentView(R.layout.go_common_activity);

		this.bindService(new Intent(GoBasicActivity.this,
				MusicPlayerService.class), mConnection,
				Context.BIND_AUTO_CREATE);

		mTts = new TextToSpeech(this, this);

		profileDBAdapter = new ProfileDBAdapter(this);
		profileDBAdapter.open();
		profileCursor = profileDBAdapter.getAllProfileCursor();
		startManagingCursor(profileCursor);
		profileCursor.moveToFirst();

		ImageButton musicPlayerBtn = (ImageButton) findViewById(R.id.music_player_btn);
		musicPlayerBtn.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				Intent musicPlayer = new Intent(GoBasicActivity.this,
						MusicListActivity.class);
				startActivity(musicPlayer);
			}
		});

		locationListener = new MyLocationListener();
		myPath = new DaligoPath();

		// 몸무게를 불러온 뒤 DB를 닫는다
		status = new Status(profileCursor
				.getInt(ProfileDBAdapter.WEIGHT_COLUMN));
		profileDBAdapter.close();

		distanceText = (TextView) findViewById(R.id.distance);
		elapsedText = (TextView) findViewById(R.id.elapsed);
		speedText = (TextView) findViewById(R.id.speed);
		caloriesText = (TextView) findViewById(R.id.calories);
		updateText();

		googleMap = (MapView) findViewById(R.id.maps);
		googleMap.setFocusable(true);
		googleMap.setSatellite(false);

		// ContextMenu 등록
		registerForContextMenu(googleMap);

		// Overlay 등록
		PathOverlay pathOverlay = new PathOverlay(googleMap, myPath, Color.RED);
		MyLocationOverlay marker = new MyLocationOverlay(this, googleMap);
		marker.enableMyLocation();

		List<Overlay> overlays = googleMap.getOverlays();
		overlays.add(pathOverlay);
		overlays.add(marker);

		// GPS Provider 선언
		lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, 5000L, 10.0f,
				locationListener);

		mc = googleMap.getController();
		mc.setZoom(18);
	}

	@Override
	protected void onDestroy() {
		// TODO Auto-generated method stub

		// handler 중지
		innerHandler.removeCallbacks(runnable);

		saveResult();
		super.onDestroy();
	}

	protected void saveResult() {
		DiaryDBAdapter diaryDBAdapter;
		GeopointDBAdapter geoDBAdapter;

		// DB에 기록하기
		Calendar calendar = Calendar.getInstance();
		Date date = calendar.getTime();
		String today = (new SimpleDateFormat("yyyyMMdd").format(date));

		int distance = status.getDistance();
		int time = status.getElapsed();
		double speed = status.getSpeed();
		double calorie = status.getCalories();

		// 달린 거리가 0보다 크면 달리기 및 좌표정보 DB에 저장
		if (distance > 0) {
			RunningData _run;
			long index = 0;
			
			_run = new RunningData(today, distance, time, speed, calorie);

			diaryDBAdapter = new DiaryDBAdapter(this);
			diaryDBAdapter.open();
			index = diaryDBAdapter.insertDiary(_run);
			diaryDBAdapter.close();

			geoDBAdapter = new GeopointDBAdapter(this);
			geoDBAdapter.open();
			geoDBAdapter.insertGeopoint(index, myPath);
			geoDBAdapter.close();
		}
	}

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		// TODO Auto-generated method stub
		innerHandler.removeCallbacks(runnable);
		lm.removeUpdates(locationListener);

		Resources res = getResources();

		// 팝업 메뉴 등록
		menu.setHeaderTitle(res.getString(R.string.pause_title));
		menu.setHeaderIcon(res.getDrawable(android.R.drawable.ic_media_pause));

		menu.add(0, PAUSE_RESUME, 0, res.getString(R.string.pause_resume));
		menu.add(0, PAUSE_SAVE, 1, res.getString(R.string.pause_save));
		menu.add(0, PAUSE_EXIT, 2, res.getString(R.string.pause_exit));

		super.onCreateContextMenu(menu, v, menuInfo);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		// TODO Auto-generated method stub
		switch (item.getItemId()) {
			case PAUSE_RESUME:
				innerHandler.removeCallbacks(runnable);
				innerHandler.postDelayed(runnable, 1000);
				lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, 5000L,
						10.0f, locationListener);
				break;

			case PAUSE_SAVE:
				Toast
						.makeText(GoBasicActivity.this, "Save",
								Toast.LENGTH_SHORT).show();
				break;

			case PAUSE_EXIT:
				finish();
				break;

			default:
				break;
		}

		return super.onContextItemSelected(item);
	}

	protected void updateText() {
		int distance = status.getDistance();

		// initialize preference sharing
		final String MYPREFS = "mySharedPreferences";
		int mode=Activity.MODE_PRIVATE;
		SharedPreferences mySharedPreferences = getSharedPreferences(MYPREFS,mode);
		// distance unit setting
		if(mySharedPreferences.getInt("Unit", 1)==1)	{
			distanceText.setText(distance + " m");
		}
		else if(mySharedPreferences.getInt("Unit", 1)==2) {
			distanceText.setText(f.format((double) distance / 1000.0) + " km");
		}
		else if(mySharedPreferences.getInt("Unit", 1)==3) {
			distanceText.setText(f.format((double) distance * 0.0006) + " mile");
		}

		int elapsed = status.getElapsed();
		int hours = elapsed / 3600;
		int minutes = (elapsed % 3600) / 60;
		int seconds = elapsed % 60;

		if (hours == 0) {
			elapsedText.setText(minutes + " m " + seconds + " s");
		} else {
			elapsedText
					.setText(hours + "h " + minutes + " m " + seconds + " s");
		}

		// �ӵ� �� Į�θ��� ���
		speedText.setText(f.format(status.getSpeed()) + " m/s");
		caloriesText.setText(f.format(status.getCalories()) + " Kcal");
		
		// Voice setting
		mTts.setLanguage(Locale.US);
		mTts.setPitch(2);
		mTts.setSpeechRate((float) 1.0);
		// voice unit setting
		int timedata;
		//greeting message
		if( elapsed==1 ){
			mTts.speak("welcome to daligo!", TextToSpeech.QUEUE_FLUSH, null);
		}
		// per 10 seconds
		else if(mySharedPreferences.getInt("voice", 1)==1){
			Information = "seconds passed!";
			if( elapsed!=0 && (elapsed%10) == 0 ){
				timedata=elapsed;
				mTts.speak(timedata+Information, TextToSpeech.QUEUE_FLUSH, null);
			}
		}
		// per 1 minutes
		else if(mySharedPreferences.getInt("voice", 1)==2){
			Information = "minutes passed!";
			if( elapsed!=0 && (elapsed%60) == 0 ){
				timedata=elapsed/60;
				mTts.speak(timedata+Information, TextToSpeech.QUEUE_FLUSH, null);
			}
		}
		// per 5 minutes
		else if(mySharedPreferences.getInt("voice", 1)==3){
			Information = "minutes passed!";
			if( elapsed!=0 && (elapsed%300) == 0 ){
				timedata=elapsed/60;
				mTts.speak(timedata+Information, TextToSpeech.QUEUE_FLUSH, null);
			}	
		}
		// per 10 minutes
		else if(mySharedPreferences.getInt("voice", 1)==4){
			Information = "minutes passed!";
			if( elapsed!=0 && (elapsed%600) == 0 ){
				timedata=elapsed/60;
				mTts.speak(timedata+Information, TextToSpeech.QUEUE_FLUSH, null);
			}	
		}
		
	}

	protected class MyLocationListener implements LocationListener {

		protected Location lastLocation = null;
		protected GeoPoint gp = null;

		@Override
		public void onLocationChanged(Location loc) {
			if (lastLocation == null) {
				outerHandler.removeCallbacks(runnable);
				outerHandler.postDelayed(runnable, 1000);
			} else {
				status.update((int) lastLocation.distanceTo(loc));
			}

			lastLocation = loc;
			gp = new GeoPoint((int) (loc.getLatitude() * 1E6), (int) (loc
					.getLongitude() * 1E6));

			mc.animateTo(gp);
			myPath.add(gp);
			googleMap.invalidate();
		}

		@Override
		public void onProviderDisabled(String provider) {
			// TODO Auto-generated method stub
		}

		@Override
		public void onProviderEnabled(String provider) {
			// TODO Auto-generated method stub
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			// TODO Auto-generated method stub
		}
	}

	protected ServiceConnection mConnection = new ServiceConnection() {
		public void onServiceConnected(ComponentName className, IBinder service) {
			mpInterface = MusicPlayerInterface.Stub
					.asInterface((IBinder) service);
			try {
				updateSongList();
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		public void onServiceDisconnected(ComponentName className) {
			mpInterface = null;
		}
	};

	public void updateSongList() throws RemoteException {
		try {
			File home = new File(MEDIA_PATH);
			File fileList[] = home.listFiles(new Mp3Filter());
			mpInterface.clearPlaylist();
			if (fileList != null) {
				for (File file : fileList) {
					songs.add(file.getName());
					mpInterface.addSongPlaylist(file.getName());
				}
			}
		} catch (DeadObjectException e) {
			Log.e(getString(R.string.app_name), e.getMessage());
		}
	}

	@Override
	public void onInit(int status) {
		// TODO Auto-generated method stub

	}
}
