package org.daligo.app;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Intent;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.os.DeadObjectException;
import android.os.IBinder;
import android.os.RemoteException;


public class MusicPlayerService extends Service {
    @Override
    public IBinder onBind(Intent intent) {
        return(mBinder);
    }
    
	private MediaPlayer mp = new MediaPlayer();
	private List<String> songs = new ArrayList<String>();
	private int currentPosition=-1;

	private NotificationManager nm;
	private static final int NOTIFY_ID = R.layout.music_list_activity;

	@Override
	public void onCreate() {
		super.onCreate();
		nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);		
	}

	@Override
	public void onDestroy() {
		mp.stop();
		mp.release();
		nm.cancel(NOTIFY_ID);
		super.onDestroy();
	}

	public IBinder getBinder() {
		return mBinder;
	}
	
	private void playSong(String file) {
		try {
			mp.reset();
			mp.setDataSource(file);
			mp.prepare();
			mp.start();
			mp.setOnCompletionListener(new OnCompletionListener() {

				public void onCompletion(MediaPlayer arg0) {
					nextSong();
				}
			});
		} catch (IOException e) {
		}
	}

	private void nextSong() {
		// Check if last song or not
		if (++currentPosition >= songs.size()) {
			currentPosition = 0;
			nm.cancel(NOTIFY_ID);
		} else {
			playSong(MusicListActivity.MEDIA_PATH + songs.get(currentPosition));
		}
	}

	private final MusicPlayerInterface.Stub mBinder = new MusicPlayerInterface.Stub() {

		public void playFile(int position) throws DeadObjectException {
			try {
				currentPosition = position;
				playSong(MusicListActivity.MEDIA_PATH + songs.get(position));

			} catch (IndexOutOfBoundsException e) {
			}
		}

		public void addSongPlaylist(String song) throws DeadObjectException {
			songs.add(song);
		}

		public void clearPlaylist() throws DeadObjectException {
			songs.clear();
		}

		public void pause() throws DeadObjectException {
			mp.pause();
		}

		public void stop() throws DeadObjectException {
			nm.cancel(NOTIFY_ID);
			mp.stop();
			currentPosition--;
		}
		
		public void play() throws DeadObjectException {
			mp.start();
		}

		@Override
		public boolean IsPlay() throws RemoteException {
			if(mp.isPlaying())
				return true;
			else
				return false;
		}
		
		public int current() throws RemoteException {
			return currentPosition;
		}
	};
}