package org.daligo.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.impl.client.DefaultHttpClient;

import android.util.Log;

/**
 * Base Rest Client.
 * Add below line to Android Manifest.
 *   - <uses-permission android:name="android.permission.INTERNET"></uses-permission>
 *
 * @author dragonkun
 */
public class RestClient {
	protected HttpClient httpclient = new DefaultHttpClient();

	public HttpResponse get(String url) {
		HttpGet httpGet = new HttpGet(url);
		HttpResponse httpResponse = null;

		try {
			httpResponse = httpclient.execute(httpGet);
			Log.i("GET", convertStreamToString(httpResponse.getEntity()
					.getContent()));
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return httpResponse;
	}

	public HttpResponse post(String url, HttpParameterMapper mapper) {
		HttpPost httpPost = new HttpPost(url);
		HttpResponse httpResponse = null;

		try {
			List<NameValuePair> nameValuePairs = mapper.mapParameter();
			httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs));
			httpResponse = httpclient.execute(httpPost);
			Log.i("POST", convertStreamToString(httpResponse.getEntity()
					.getContent()));
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return httpResponse;
	}

	public HttpResponse put(String url, HttpParameterMapper mapper) {
		HttpPut httpPut = new HttpPut(url);
		HttpResponse httpResponse = null;

		try {
			List<NameValuePair> nameValuePairs = mapper.mapParameter();
			httpPut.setEntity(new UrlEncodedFormEntity(nameValuePairs));
			httpResponse = httpclient.execute(httpPut);
			Log.i("PUT", convertStreamToString(httpResponse.getEntity()
					.getContent()));
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return httpResponse;
	}

	public HttpResponse delete(String url) {
		HttpDelete httpDelete = new HttpDelete(url);
		HttpResponse httpResponse = null;

		try {
			httpResponse = httpclient.execute(httpDelete);
			Log.i("DELETE", convertStreamToString(httpResponse.getEntity()
					.getContent()));
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return httpResponse;
	}

	private static String convertStreamToString(InputStream is) {		
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		StringBuilder sb = new StringBuilder();
		String line = null;
		
		try {
			while ((line = reader.readLine()) != null) {
				sb.append(line + "\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				is.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return sb.toString();
	}

}