package org.daligo.domain;

import org.daligo.type.RunningData;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.Log;

public class DiaryDBAdapter {

	private static final String DATABASE_NAME = "daligo.db";
	private static final String DATABASE_TABLE = "diary";
	private static final int DATABASE_VERSION = 1;

	private SQLiteDatabase db;
	private final Context context;

	private diaryDBOpenHelper dbHelper;

	public static final String KEY_ID = "_id";
	public static final int ID_COLUMN = 0;

	public static final String KEY_DATE = "date";
	public static final int DATE_COLUMN = 1;

	public static final String KEY_DISTANCE = "distance";
	public static final int DISTANCE_COLUMN = 2;

	public static final String KEY_TIME = "time";
	public static final int TIME_COLUMN = 3;

	public static final String KEY_SPEED = "speed";
	public static final int SPEED_COLUMN = 4;

	public static final String KEY_CALORIE = "calorie";
	public static final int CALORIE_COLUMN = 5;

	public DiaryDBAdapter(Context _context) {
		this.context = _context;
		dbHelper = new diaryDBOpenHelper(context, DATABASE_NAME, null,
				DATABASE_VERSION);
	}

	private static class diaryDBOpenHelper extends SQLiteOpenHelper {
		public diaryDBOpenHelper(Context context, String name,
				CursorFactory factory, int version) {
			super(context, name, factory, version);
			// TODO Auto-generated constructor stub
		}

		// 새로운 데이터베이스를 만드는 sql문
		private static final String DATABASE_CREATE = "create table if not exists "
				+ DATABASE_TABLE
				+ " ("
				+ KEY_ID
				+ " integer primary key autoincrement, "
				+ KEY_DATE
				+ " date not null, "
				+ KEY_DISTANCE
				+ " real, "
				+ KEY_TIME
				+ " real, " + KEY_SPEED + " real," + KEY_CALORIE + " real); ";

		@Override
		public void onCreate(SQLiteDatabase _db) {
			// TODO Auto-generated method stub
			_db.execSQL(DATABASE_CREATE);
		}

		@Override
		public void onUpgrade(SQLiteDatabase _db, int _oldVersion,
				int _newVersion) {
			// TODO Auto-generated method stub
			Log.w("DiaryDBAdapter", "Upgrading from version " + _oldVersion
					+ " to " + _newVersion
					+ ", which will destroy all old data");
			// 기존 테이블을 버린다.
			_db.execSQL("DROP TABALE IF EXISTS " + DATABASE_TABLE);
			// 새것을 만든다
			onCreate(_db);
		}
	}

	public void close() {
		db.close();
	}

	public void open() throws SQLiteException {
		try {
			db = dbHelper.getWritableDatabase();
		} catch (SQLiteException ex) {
			db = dbHelper.getReadableDatabase();
		}
	}

	// 새 달리기 기록을 삽입한다
	public long insertDiary(RunningData _running) {
		ContentValues dataValues = new ContentValues();

		dataValues.put(KEY_DATE, _running.getDate());
		dataValues.put(KEY_DISTANCE, _running.getDistance());
		dataValues.put(KEY_TIME, _running.getTime());
		dataValues.put(KEY_SPEED, _running.getSpeed());
		dataValues.put(KEY_CALORIE, _running.getCalorie());

		// 이 행을 삽입한다
		return db.insert(DATABASE_TABLE, null, dataValues);
	}

	// 기록을 제거한다.
	public boolean removeDiary(long _rowIndex) {
		return db.delete(DATABASE_TABLE, KEY_ID + "=" + _rowIndex, null) > 0;
	}

	public Cursor getAllDiaryCursor() {
		return db.query(DATABASE_TABLE, new String[] { KEY_ID, KEY_DATE,
				KEY_DISTANCE, KEY_TIME, KEY_SPEED, KEY_CALORIE }, null, null,
				null, null, null);

	}

	public Cursor setCursorDiary(long _rowIndex) throws SQLException {
		Cursor result = db.query(true, DATABASE_TABLE, new String[] { KEY_ID,
				KEY_DATE, KEY_DISTANCE, KEY_TIME, KEY_SPEED, KEY_CALORIE },
				KEY_ID + "=" + _rowIndex, null, null, null, null, null);
		if ((result.getCount() == 0) || !result.moveToFirst()) {
			throw new SQLException("No to do items found for row: " + _rowIndex);
		}
		return result;
	}

	public RunningData getRunningData(long _rowIndex) throws SQLException {
		Cursor cursor = db.query(true, DATABASE_TABLE, new String[] { KEY_ID,
				KEY_DATE, KEY_DISTANCE, KEY_TIME, KEY_SPEED, KEY_CALORIE },
				KEY_ID + "=" + _rowIndex, null, null, null, null, null);
		if ((cursor.getCount() == 0) || !cursor.moveToFirst()) {
			throw new SQLException("No to do items found for row: " + _rowIndex);
		}

		String date;
		int distance;
		int time;
		double speed;
		double calorie;

		date = cursor.getString(DATE_COLUMN);
		distance = cursor.getInt(DISTANCE_COLUMN);
		time = cursor.getInt(TIME_COLUMN);
		speed = cursor.getDouble(SPEED_COLUMN);
		calorie = cursor.getDouble(CALORIE_COLUMN);

		RunningData result = new RunningData(date, distance, time, speed,
				calorie);
		return result;
	}
}
