package org.daligo.domain;


import org.daligo.type.DaligoPath;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.Log;

public class GeopointDBAdapter {
	private static final String DATABASE_NAME = "daligo.db";
	private static final String DATABASE_TABLE = "geopoint";
	private static final int DATABASE_VERSION = 1;

	private SQLiteDatabase db;
	private final Context context;

	private geopointDBOpenHelper dbHelper;

	public static final String KEY_ID = "_id";
	public static final int ID_COLUMN = 0;

	public static final String KEY_LATITUDE = "latitude";
	public static final int LATITUDE_COLUMN = 1;

	public static final String KEY_LONGITUDE = "longitude";
	public static final int LONGITUDE_COLUMN = 2;

	public static final String KEY_INDEX = "data_id";
	public static final int INDEX_COLUMN = 3;

	public GeopointDBAdapter(Context _context) {
		this.context = _context;
		dbHelper = new geopointDBOpenHelper(context, DATABASE_NAME, null,
				DATABASE_VERSION);
	}

	private static class geopointDBOpenHelper extends SQLiteOpenHelper {
		public geopointDBOpenHelper(Context context, String name,
				CursorFactory factory, int version) {
			super(context, name, factory, version);
			// TODO Auto-generated constructor stub
		}

		// 새로운 데이터베이스를 만드는 sql문
		private static final String DATABASE_CREATE = "create table if not exists "
			+ DATABASE_TABLE + " (" + KEY_ID
			+ " integer primary key autoincrement, " + KEY_LATITUDE
			+ " real, " + KEY_LONGITUDE + " real, " + KEY_INDEX
			+ " integer) ";

		@Override
		public void onCreate(SQLiteDatabase _db) {
			// TODO Auto-generated method stub
			_db.execSQL(DATABASE_CREATE);
		}

		@Override
		public void onUpgrade(SQLiteDatabase _db, int _oldVersion,
				int _newVersion) {
			// TODO Auto-generated method stub
			Log.w("GeopointDBAdapter", "Upgrading from version " + _oldVersion
					+ " to " + _newVersion
					+ ", which will destroy all old data");
			// 기존 테이블을 버린다.
			_db.execSQL("DROP TABALE IF EXISTS " + DATABASE_TABLE);
			// 새것을 만든다
			onCreate(_db);
		}
	}

	public void close() {
		db.close();
	}

	public void open() throws SQLiteException {
		try {
			db = dbHelper.getWritableDatabase();
		} catch (SQLiteException ex) {
			db = dbHelper.getReadableDatabase();
		}
	}

	// 새 geopoint 기록을 삽입한다
	public long insertGeopoint(long index, DaligoPath _path) {
		ContentValues dataValues = new ContentValues();
		int size = _path.size();
		long _rowId = 0;
		
		for (int i = 0 ; i < size ; i++){
			dataValues.put(KEY_LATITUDE, _path.get(i).getLatitudeE6());
			dataValues.put(KEY_LONGITUDE, _path.get(i).getLongitudeE6());
			dataValues.put(KEY_INDEX, index);
			// 이 행을 삽입한다
			_rowId = db.insert(DATABASE_TABLE, null, dataValues);
		}	
		
		return _rowId;
	}

	// 기록을 제거한다.
	public boolean removeGeopoint(long _rowIndex) {
		return db.delete(DATABASE_TABLE, KEY_ID + "=" + _rowIndex, null) > 0;
	}

	public Cursor getAllGeopointCursor() {
		return db.query(DATABASE_TABLE, new String[] { KEY_ID, KEY_LATITUDE,
				KEY_LONGITUDE, KEY_INDEX}, null, null,
				null, null, null);

	}

	public Cursor setCursorGeopoint(long _rowIndex) throws SQLException {
		Cursor result = db.query(true, DATABASE_TABLE, new String[] { KEY_ID,
				KEY_LATITUDE, KEY_LONGITUDE, KEY_INDEX },
				KEY_ID + "=" + _rowIndex, null, null, null, null, null);
		if ((result.getCount() == 0) || !result.moveToFirst()) {
			throw new SQLException("No to do items found for row: " + _rowIndex);
		}
		return result;
	}
}
